# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#
# This file was automatically built from t/z9-read-variations.ttmpl
#
# Do not edit this file, instead edit the template and rebuild by running
# t/build-test-scripts
#
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
use strict;
use warnings;

# Try many combinations of read operations on an IO::Callback, checking that each
# gives exactly the same results as Perl does for a real file.

# This test is split into several .t files (one for each $seglen value), since the
# large number of tests uses too much memory if run as a single .t file. 

our $seglen;
$seglen = 3;

use lib 't';
require 'read-variations.inc';

